@@ -10,37 +10,36 @@ verbose = TRUE,
 ...)
 {
   type <- match.arg(type)
-  if (class(object) != "pcrfit" && class(object) != "nls") stop("Use only with objects of class 'pcrfit' or 'nls'!")   
+  if (class(object) != "pcrfit") stop("Use only with objects of class 'pcrfit'!")   
     
-  DATA <- eval(object$data, sys.frame(0))    
-  if (class(object) != "pcrfit") resp <- all.vars(formula(object)[[2]]) else resp <- 2
+  fetchDATA <- fetchData(object)
+  DATA <- fetchDATA$data
+  PRED.pos <- fetchDATA$pred.pos
+  RESP.pos <- fetchDATA$resp.pos
+  PRED.name <- fetchDATA$pred.name
   
-  fitted1 <- fitted(object)
+  fitted1 <- fitted(object)    
   resid1 <- residuals(object)   
   modList <- list()
   effList <- list()    
   ndata <- nrow(DATA)  
-  noconv <- 0
-  if (class(object) != "pcrfit") do.eff = FALSE
-  
-  ### collect stats and if 'pcrfit' model, 
-  ### efficiency analysis     
+  noconv <- 0     
   
   for (i in 1:B) {    
     newDATA <- DATA         
-    if (type == "boot") newDATA[, resp] <- fitted1 + sample(scale(resid1, scale = FALSE), replace = TRUE)
-     else {
+    if (type == "boot") newDATA[, RESP.pos] <- fitted1 + sample(scale(resid1, scale = FALSE), replace = TRUE)
+    else {
       sampleVec <- sample(1:ndata, njack)    
       newDATA <- newDATA[-sampleVec, ]      
-     }
-     
-    newMODEL <- try(update(object, data = newDATA))
+    }                
+  
+    newMODEL <- try(update(object, data = newDATA))      
     
     if (inherits(newMODEL, "try-error")) {
       noconv <- noconv + 1
       next
-    }
-          
+    }                
+    
     if (plot) plot(newMODEL, ...)
     
     modList[[i]] <- list(coef = coef(newMODEL), sigma = summary(newMODEL)$sigma,
@@ -48,18 +47,12 @@ verbose = TRUE,
                          dfb = abs(coef(newMODEL) - coef(object))/(summary(object)$parameters[, 2]),
                          gof = pcrGOF(newMODEL)) 
     
-    if (do.eff) effList[[i]] <- efficiency(newMODEL, plot = FALSE, ...)          
-    
-    if (verbose) {
-      if (i %% 10 == 0) cat(i) else cat(".")
-      if (i %% 50 == 0) cat("\n")
-      flush.console()
-    }
+    if (do.eff) effList[[i]] <- efficiency(newMODEL, plot = FALSE, ...)[c(1, 7:18)]     
+    if (verbose) counter(i)    
   }
   cat("\n\n")
-  if (verbose) cat("fitting converged in ", 100 - (noconv/B), "% of iterations.\n\n", sep = "") 
-   
-  ### stat analysis    
+  if (verbose) cat("fitting converged in ", 100 - (noconv/B), "% of iterations.\n\n", sep = "")      
+  
   COEF <- t(sapply(modList, function(z) z$coef))  
   RSE <- sapply(modList, function(z) z$sigma)  
   RSS <- sapply(modList, function(z) z$rss)           
